@@ -4,6 +4,7 @@ import json
 import time
 import threading
 import six
+import ssl
 from six.moves import queue
 
 import websocket
@@ -90,6 +91,8 @@ class WAMPClient(threading.Thread):
     authid = None
     authmethods = None
     timeout = None
+    sslopt = None
+    sockopt = None
 
     auto_reconnect = True
 
@@ -114,7 +117,9 @@ class WAMPClient(threading.Thread):
                 authmethods=None,
                 authid=None,
                 timeout=10,
-                auto_reconnect=True,
+                auto_reconnect=1,
+                sslopt=None,
+                sockopt=None,
                 ):
 
         self._state = STATE_DISCONNECTED
@@ -122,6 +127,8 @@ class WAMPClient(threading.Thread):
         super(WAMPClient,self).__init__()
         self.daemon = True
         self._request_loop_notify_restart = threading.Condition()
+        if auto_reconnect == True:
+            auto_reconnect = 1
         self.configure(
             url = url,
             uri_base = uri_base,
@@ -131,6 +138,8 @@ class WAMPClient(threading.Thread):
             authid = authid,
             authmethods = authmethods,
             auto_reconnect = auto_reconnect,
+            sslopt = sslopt,
+            sockopt = sockopt,
         )
 
     def get_full_uri(self,uri):
@@ -140,11 +149,6 @@ class WAMPClient(threading.Thread):
             return self.uri_base + '.' + uri
         return uri
 
-    def generate_request_id(self):
-        """ We cheat, we just use the millisecond timestamp for the request
-        """
-        return int(round(time.time() * 1000))
-
     def connect(self,soft_reset=False,**options):
         """ This just creates the websocket connection
         """
@@ -154,6 +158,7 @@ class WAMPClient(threading.Thread):
         m = re.search('(ws+)://([\w\.]+)(:?:(\d+))?',self.url)
 
         options['subprotocols'] = ['wamp.2.json']
+        options['enable_multithread'] = True
         options.setdefault('timeout',self._loop_timeout)
 
         # Handle the weird issue in websocket that the origin
@@ -169,6 +174,21 @@ class WAMPClient(threading.Thread):
         # case we try and try again...
         while True:
             try:
+                if self.sslopt:
+                    options.setdefault('sslopt',self.sslopt)
+
+                # By default if no settings are chosen we apply
+                # the looser traditional policy (makes life less
+                # secure but less excruciating on windows)
+                if options.get('sslopt') is None:
+                    options['sslopt'] = {
+                        "cert_reqs":ssl.CERT_NONE,
+                        "check_hostname": False
+                    }
+
+                if self.sockopt:
+                    options.setdefault('sockopt',self.sockopt)
+
                 self.ws = websocket.create_connection(
                                 self.url,
                                 **options
@@ -176,8 +196,14 @@ class WAMPClient(threading.Thread):
                 self.handle_connect()
             except Exception as ex:
                 if self.auto_reconnect:
-                    # FIXME: how long to wait?
-                    time.sleep(1)
+                    logger.debug(
+                        "Error connecting to {url}. Reconnection attempt in {retry} second(s). {err}".format(
+                            url=self.url,
+                            retry=self.auto_reconnect,
+                            err=ex
+                        )
+                    )
+                    time.sleep(self.auto_reconnect)
                     continue
                 else:
                     raise
@@ -208,7 +234,7 @@ class WAMPClient(threading.Thread):
     def configure(self, **kwargs):
         for k in ('url','uri_base','realm',
                   'agent','timeout','authmethods', 'authid',
-                  'auto_reconnect'):
+                  'auto_reconnect', 'sslopt', 'sockopt'):
             if k in kwargs:
                 setattr(self,k,kwargs[k])
 
@@ -442,7 +468,6 @@ class WAMPClient(threading.Thread):
     def subscribe(self,topic,callback=None,options=None):
         """ Subscribe to a uri for events from a publisher
         """
-        id = self.generate_request_id()
         full_topic = self.get_full_uri(topic)
         result = self.send_and_await_response(SUBSCRIBE(
                                     options={},
@@ -456,26 +481,27 @@ class WAMPClient(threading.Thread):
     def publish(self,topic,options=None,args=None,kwargs=None):
         """ Publishes a messages to the server
         """
-        id = self.generate_request_id()
         topic = self.get_full_uri(topic)
         if options is None:
             options = {'acknowledge':True}
         if options.get('acknowledge'):
-            result = self.send_and_await_response(PUBLISH(
+            request = PUBLISH(
                         options=options or {},
                         topic=topic,
                         args=args or [],
                         kwargs=kwargs or {}
-                      ))
+                      )
+            result = self.send_and_await_response(request)
             return result
         else:
-            self.send_message(PUBLISH(
+            request = PUBLISH(
                         options=options or {},
                         topic=topic,
                         args=args or [],
                         kwargs=kwargs or {}
-                      ))
-            return id
+                      )
+            self.send_message(request)
+            return request.request_id
 
     def disconnect(self):
         """ Disconnect from the websocket and pause the process
@@ -613,6 +639,9 @@ class WAMPClient(threading.Thread):
                     # FIXME: need to randomly wait
                     time.sleep(1)
                     if not data: continue
+            except Exception as ex:
+                logger.error("ERROR in main loop: {}".format(ex))
+                continue
 
             try:
                 logger.debug("<RCV: {}".format(data))
@@ -669,4 +698,3 @@ class WAMPClientTicket(WAMPClient):
             signature = self.password,
             extra = {}
         ))
-
